// MagnifierView.cpp : implementation of the CMagnifierView class
//

#include "stdafx.h"
#include "Magnifier.h"

#include "MagnifierDoc.h"
#include "MagnifierView.h"

#include "AutoPen.h"
#include "BitmapDC.h"
#include "memdc.h"
#include "MaskedBitmap.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CMagnifierView, CScrollView)

BEGIN_MESSAGE_MAP(CMagnifierView, CScrollView)
	//{{AFX_MSG_MAP(CMagnifierView)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	ON_WM_SETCURSOR()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMagnifierView construction/destruction

CMagnifierView::CMagnifierView()
{
	LPVOID lpvResource = (LPVOID) ::LoadResource(NULL,
        ::FindResource(NULL, MAKEINTRESOURCE(IDB_BITMAP1),
        RT_BITMAP));
    m_dib.AttachMemory(lpvResource);
}

CMagnifierView::~CMagnifierView()
{
}

BOOL CMagnifierView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMagnifierView drawing

void CMagnifierView::OnDraw(CDC* pDC)
{
	m_dib.Draw(pDC, CPoint(0, 0), m_dib.GetDimensions());
	SetScrollSizes(MM_TEXT, m_dib.GetDimensions());
}

/////////////////////////////////////////////////////////////////////////////
// CMagnifierView printing

BOOL CMagnifierView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMagnifierView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMagnifierView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CMagnifierView diagnostics

#ifdef _DEBUG
void CMagnifierView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CMagnifierView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CMagnifierDoc* CMagnifierView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMagnifierDoc)));
	return (CMagnifierDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMagnifierView message handlers

void CMagnifierView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCursor(AfxGetApp()->LoadCursor(IDC_CURSOR1));
	static char BASED_CODE szFilter[] = "Bitmap Files (*.bmp)|*.bmp|";
	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY, szFilter, this);
	if(dlg.DoModal() == IDOK)
	{
		CFile file(dlg.GetPathName(), CFile::modeRead);
		if(m_dib.Read(&file) == TRUE)
			Invalidate();
	}
	CScrollView::OnLButtonDown(nFlags, point);
}

void CMagnifierView::OnMouseMove(UINT nFlags, CPoint point) 
{
	SetCursor(AfxGetApp()->LoadCursor(IDC_CURSOR1));
	CPoint p = GetScrollPosition();
	CClientDC pdc(this);
	CMemDC dc(&pdc);
	m_dib.Draw(&dc, CPoint(-p.x, -p.y), m_dib.GetDimensions());
	CRect rect1(CPoint(0, 0), m_dib.GetDimensions());
	CRect sel;
	CMaskedBitmap* bitm;
	CMaskedBitmap* bit;
	if(rect1.PtInRect(point))
	{
		CRgn rgn1, rgn2;
		sel = CRect(point.x-(20*1), point.y-(20*1), point.x+(20*1), point.y+(20*1));
		CBitmapDC bitmap(40*1, 40*1, &dc);
		CBitmapDC bitmap2(120*1, 120*1, &dc);
		rgn1.CreateEllipticRgn(0, 0, 120, 120);
		rgn2.CreateRectRgn(0, 0, 120, 120);
		rgn1.CombineRgn(&rgn2, &rgn1, RGN_DIFF);
		int y = sel.top, x = sel.left;
		for(int i = y; i<y+(40*1);i++)
		{
			for(int j = x; j<x+(40*1); j++)
			{
				COLORREF color = dc.GetPixel(j, i);
				int red = GetRValue(color);
				int green = GetGValue(color);
				int blue = GetBValue(color);
				bitmap.SetPixel(j-x, i-y, color);
			}
		}
		bitm = bitmap.Close();
		CDC dcMem;
		dcMem.CreateCompatibleDC(&dc);
		dcMem.SelectObject(bitm);
		bitmap2.StretchBlt(0, 0, 120*1, 120*1, &dcMem, 0, 0, 40, 40, SRCCOPY);
		bitmap2.FillRgn(&rgn1, &CBrush(RGB(255, 0, 254)));
		bit = bitmap2.Close();
		dc.SelectObject(&CPen(PS_SOLID, 1, RGB(0, 0, 0)));
		dc.SelectObject(&CBrush(NULL, RGB(0, 0, 0)));
		dc.Ellipse(point.x-(60*1)-1, point.y-(60*1)-1, point.x+(60*1), point.y+(60*1));
		bit->DrawTransparent(&dc, point.x-(60*1), point.y-(60*1), RGB(255, 0, 254));
		dc.MoveTo(point.x, point.y-(60*1)-1);
		dc.LineTo(point.x, point.y-(10*1)+1);
		dc.MoveTo(point.x, point.y+(10*1)-1);
		dc.LineTo(point.x, point.y+(60*1)+1);
		dc.MoveTo(point.x-(60*1)-1, point.y);
		dc.LineTo(point.x-(10*1)+1, point.y);		
		dc.MoveTo(point.x+(10*1)-1, point.y);
		dc.LineTo(point.x+(60*1)+1, point.y);		
	}
	CScrollView::OnMouseMove(nFlags, point);
}

BOOL CMagnifierView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	return CScrollView::OnEraseBkgnd(pDC);
}

BOOL CMagnifierView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	SetCursor(AfxGetApp()->LoadCursor(IDC_CURSOR1));
	return CScrollView::OnSetCursor(pWnd, nHitTest, message);
}

void CMagnifierView::OnFileOpen() 
{
	SetCursor(AfxGetApp()->LoadCursor(IDC_CURSOR1));
	static char BASED_CODE szFilter[] = "Bitmap Files (*.bmp)|*.bmp|";
	CFileDialog dlg(TRUE, NULL, NULL, OFN_HIDEREADONLY, szFilter, this);
	if(dlg.DoModal() == IDOK)
	{
		CFile file(dlg.GetPathName(), CFile::modeRead);
		if(m_dib.Read(&file) == TRUE)
			Invalidate();
	}
}

